@@ -721,22 +721,38 @@ struct
         | End_of_file -> Lwt.return_none
         | exn -> Lwt.fail exn)
 
-  let unsafe_read_into ic buf ofs len =
+  let unsafe_read_into' ic blit buf ofs len =
     let avail = ic.max - ic.ptr in
     if avail > 0 then begin
       let len = min len avail in
-      Lwt_bytes.unsafe_blit_to_bytes ic.buffer ic.ptr buf ofs len;
+      blit ic.buffer ic.ptr buf ofs len;
       ic.ptr <- ic.ptr + len;
       Lwt.return len
     end else begin
       refill ic >>= fun n ->
       let len = min len n in
-      Lwt_bytes.unsafe_blit_to_bytes ic.buffer 0 buf ofs len;
+      blit ic.buffer 0 buf ofs len;
       ic.ptr <- len;
       ic.max <- n;
       Lwt.return len
     end
 
+  let unsafe_read_into_bigstring ic buf ofs len =
+    unsafe_read_into' ic Lwt_bytes.unsafe_blit buf ofs len
+
+  let unsafe_read_into ic buf ofs len =
+    unsafe_read_into' ic Lwt_bytes.unsafe_blit_to_bytes buf ofs len
+
+  let read_into_bigstring ic buf ofs len =
+    if ofs < 0 || len < 0 || ofs + len > Lwt_bytes.length buf then
+      Lwt.fail (Invalid_argument "Lwt_io.read_into_bigstring")
+    else begin
+      if len = 0 then
+        Lwt.return 0
+      else
+        unsafe_read_into_bigstring ic buf ofs len
+    end
+
   let read_into ic buf ofs len =
     if ofs < 0 || len < 0 || ofs + len > Bytes.length buf then
       Lwt.fail (Invalid_argument "Lwt_io.read_into")
@@ -747,16 +763,36 @@ struct
         unsafe_read_into ic buf ofs len
     end
 
-  let rec unsafe_read_into_exactly ic buf ofs len =
-    unsafe_read_into ic buf ofs len >>= function
-    | 0 ->
-      Lwt.fail End_of_file
-    | n ->
-      let len = len - n in
+  let unsafe_read_into_exactly' read_into ic buf ofs len =
+    let rec loop ic buf ofs len =
+      read_into ic buf ofs len >>= function
+      | 0 ->
+          Lwt.fail End_of_file
+      | n ->
+          let len = len - n in
+          if len = 0 then
+            Lwt.return_unit
+          else
+            loop ic buf (ofs + n) len
+    in
+    loop ic buf ofs len
+
+
+  let unsafe_read_into_exactly_bigstring ic buf ofs len =
+    unsafe_read_into_exactly' unsafe_read_into_bigstring ic buf ofs len
+
+  let unsafe_read_into_exactly ic buf ofs len =
+    unsafe_read_into_exactly' unsafe_read_into ic buf ofs len
+
+  let read_into_exactly_bigstring ic buf ofs len =
+    if ofs < 0 || len < 0 || ofs + len > Lwt_bytes.length buf then
+      Lwt.fail (Invalid_argument "Lwt_io.read_into_exactly_bigstring")
+    else begin
       if len = 0 then
         Lwt.return_unit
       else
-        unsafe_read_into_exactly ic buf (ofs + n) len
+        unsafe_read_into_exactly_bigstring ic buf ofs len
+    end
 
   let read_into_exactly ic buf ofs len =
     if ofs < 0 || len < 0 || ofs + len > Bytes.length buf then
@@ -832,14 +868,14 @@ struct
       flush_partial oc >>= fun _ ->
       write_char oc ch
 
-  let rec unsafe_write_from oc str ofs len =
+  let rec unsafe_write_from' blit oc str ofs len =
     let avail = oc.length - oc.ptr in
     if avail >= len then begin
-      Lwt_bytes.unsafe_blit_from_bytes str ofs oc.buffer oc.ptr len;
+      blit str ofs oc.buffer oc.ptr len;
       oc.ptr <- oc.ptr + len;
       Lwt.return 0
     end else begin
-      Lwt_bytes.unsafe_blit_from_bytes str ofs oc.buffer oc.ptr avail;
+      blit str ofs oc.buffer oc.ptr avail;
       oc.ptr <- oc.length;
       flush_partial oc >>= fun _ ->
       let len = len - avail in
@@ -848,13 +884,30 @@ struct
           Lwt.return 0
         else
           (* Everything has been written, try to write more: *)
-          unsafe_write_from oc str (ofs + avail) len
+          unsafe_write_from' blit oc str (ofs + avail) len
       end else
         (* Not everything has been written, just what is
            remaining: *)
         Lwt.return len
     end
 
+  let unsafe_write_from_bigstring oc bytes ofs len =
+    unsafe_write_from' Lwt_bytes.blit oc bytes ofs len
+
+  let unsafe_write_from oc str ofs len =
+    unsafe_write_from' Lwt_bytes.unsafe_blit_from_bytes oc str ofs len
+
+  let write_from_bigstring oc bytes ofs len =
+    if ofs < 0 || len < 0 || ofs + len > Lwt_bytes.length bytes then
+      Lwt.fail (Invalid_argument "Lwt_io.write_from_bigstring")
+    else begin
+      if len = 0 then
+        Lwt.return 0
+      else
+        unsafe_write_from_bigstring oc bytes ofs len >>= fun remaining ->
+        Lwt.return (len - remaining)
+    end
+
   let write_from oc buf ofs len =
     if ofs < 0 || len < 0 || ofs + len > Bytes.length buf then
       Lwt.fail (Invalid_argument "Lwt_io.write_from")
@@ -870,12 +923,21 @@ struct
     let buf = Bytes.unsafe_of_string buf in
     write_from oc buf ofs len
 
-  let rec unsafe_write_from_exactly oc buf ofs len =
-    unsafe_write_from oc buf ofs len >>= function
-    | 0 ->
-      Lwt.return_unit
-    | n ->
-      unsafe_write_from_exactly oc buf (ofs + len - n) n
+  let unsafe_write_from_exactly' write_from oc buf ofs len =
+    let rec loop oc buf ofs len =
+      write_from oc buf ofs len >>= function
+      | 0 ->
+        Lwt.return_unit
+      | n ->
+          loop oc buf (ofs + len - n) n
+    in
+    loop oc buf ofs len
+
+  let unsafe_write_from_exactly oc buf ofs len =
+    unsafe_write_from_exactly' unsafe_write_from oc buf ofs len
+
+  let unsafe_write_from_exactly_bigstring oc buf ofs len =
+    unsafe_write_from_exactly' unsafe_write_from_bigstring oc buf ofs len
 
   let write_from_exactly oc buf ofs len =
     if ofs < 0 || len < 0 || ofs + len > Bytes.length buf then
@@ -887,6 +949,16 @@ struct
         unsafe_write_from_exactly oc buf ofs len
     end
 
+  let write_from_exactly_bigstring oc buf ofs len =
+    if ofs < 0 || len < 0 || ofs + len > Lwt_bytes.length buf then
+      Lwt.fail (Invalid_argument "Lwt_io.write_from_exactly_bigstring")
+    else begin
+      if len = 0 then
+        Lwt.return_unit
+      else
+        unsafe_write_from_exactly_bigstring oc buf ofs len
+    end
+
   let write_from_string_exactly oc buf ofs len =
     let buf = Bytes.unsafe_of_string buf in
     write_from_exactly oc buf ofs len
@@ -1139,6 +1211,12 @@ let read_into ic str ofs len =
 let read_into_exactly ic str ofs len =
   primitive (fun ic -> Primitives.read_into_exactly ic str ofs len) ic
 
+let read_into_bigstring ic bytes ofs len =
+  primitive (fun ic -> Primitives.read_into_bigstring ic bytes ofs len) ic
+
+let read_into_exactly_bigstring ic bytes ofs len =
+  primitive (fun ic -> Primitives.read_into_exactly_bigstring ic bytes ofs len) ic
+
 let read_value ic =
   primitive Primitives.read_value ic
 
@@ -1169,12 +1247,18 @@ let write_line oc x =
 let write_from oc str ofs len =
   primitive (fun oc -> Primitives.write_from oc str ofs len) oc
 
+let write_from_bigstring oc bytes ofs len =
+  primitive (fun oc -> Primitives.write_from_bigstring oc bytes ofs len) oc
+
 let write_from_string oc str ofs len =
   primitive (fun oc -> Primitives.write_from_string oc str ofs len) oc
 
 let write_from_exactly oc str ofs len =
   primitive (fun oc -> Primitives.write_from_exactly oc str ofs len) oc
 
+let write_from_exactly_bigstring oc bytes ofs len =
+  primitive (fun oc -> Primitives.write_from_exactly_bigstring oc bytes ofs len) oc
+
 let write_from_string_exactly oc str ofs len =
   primitive (fun oc -> Primitives.write_from_string_exactly oc str ofs len) oc
 
@@ -1386,6 +1470,34 @@ let create_temp_dir
   in
   attempt 0
 
+let win32_unlink fn =
+  Lwt.catch
+    (fun () -> Lwt_unix.unlink fn)
+    (function
+     | Unix.Unix_error (Unix.EACCES, _, _) as exn ->
+       (* Try removing the read-only attribute before retrying unlink. We catch
+          any exception here and ignore it in favour of the original [exn]. *)
+       Lwt.catch
+         (fun () ->
+           Lwt_unix.lstat fn >>= fun {st_perm; _} ->
+           Lwt_unix.chmod fn 0o666 >>= fun () ->
+           Lwt.catch
+             (fun () -> Lwt_unix.unlink fn)
+             (function _ ->
+                (* If everything succeeded but the final removal still failed,
+                   restore original permissions *)
+                Lwt_unix.chmod fn st_perm >>= fun () ->
+                Lwt.fail exn)
+         )
+         (fun _ -> Lwt.fail exn)
+     | exn -> Lwt.fail exn)
+
+let unlink =
+  if Sys.win32 then
+    win32_unlink
+  else
+    Lwt_unix.unlink
+
 (* This is likely VERY slow for directories with many files. That is probably
    best addressed by switching to blocking calls run inside a worker thread,
    i.e. with Lwt_preemptive. *)
@@ -1397,11 +1509,16 @@ let rec delete_recursively directory =
       Lwt.return ()
     else
       let path = Filename.concat directory entry in
-      Lwt_unix.lstat path >>= fun stat ->
-      if stat.Lwt_unix.st_kind = Lwt_unix.S_DIR then
-        delete_recursively path
-      else
-        Lwt_unix.unlink path
+      Lwt_unix.lstat path >>= fun {Lwt_unix.st_kind; _} ->
+      match st_kind with
+      | S_DIR -> delete_recursively path
+      | S_LNK when (Sys.win32 || Sys.cygwin) ->
+         Lwt_unix.stat path >>= fun {Lwt_unix.st_kind; _} ->
+         begin match st_kind with
+         | S_DIR -> Lwt_unix.rmdir path
+         | _ -> unlink path
+         end
+      | _ -> unlink path
   end >>= fun () ->
   Lwt_unix.rmdir directory
 
@@ -1513,8 +1630,14 @@ let establish_server_generic
 
   let rec accept_loop () =
     let try_to_accept =
-      Lwt_unix.accept listening_socket >|= fun x ->
-      `Accepted x
+      Lwt.catch
+        (fun () ->
+           Lwt_unix.accept listening_socket >|= fun x ->
+           `Accepted x)
+        (function
+          | Unix.Unix_error (Unix.ECONNABORTED, _, _) ->
+            Lwt.return `Try_again
+          | e -> Lwt.fail e)
     in
 
     Lwt.pick [try_to_accept; should_stop] >>= function
@@ -1540,6 +1663,8 @@ let establish_server_generic
 
       Lwt.wakeup_later notify_listening_socket_closed ();
       Lwt.return_unit
+    | `Try_again ->
+      accept_loop ()
   in
 
   let server =
