@@ -245,10 +245,11 @@ val read_lines : input_channel -> string Lwt_stream.t
   (** [read_lines ic] returns a stream holding all lines of [ic] *)
 
 val read : ?count : int -> input_channel -> string Lwt.t
-(** If [~count] is specified, [read ~count ic] reads at most [~count] characters
-    from [ic]. Note that fewer than [~count] characters can be read; check the
-    size of the resulting string. [read] returns [""] if the end of input is
-    reached.
+(** If [~count] is specified, [read ~count ic] reads at most [~count] bytes from
+    [ic] in one read operation. Note that fewer than [~count] bytes can be read.
+    This can happen for multiple reasons, including end of input, or no more
+    data currently available. Check the size of the resulting string. [read]
+    resolves with [""] if the input channel is already at the end of input.
 
     If [~count] is not specified, [read ic] reads all bytes until the end of
     input. *)
@@ -267,15 +268,19 @@ val read_into_exactly : input_channel -> bytes -> int -> int -> unit Lwt.t
 
       @raise End_of_file on end of input *)
 
+val read_into_bigstring : input_channel -> Lwt_bytes.t -> int -> int -> int Lwt.t
+
+val read_into_exactly_bigstring : input_channel -> Lwt_bytes.t -> int -> int -> unit Lwt.t
+
 val read_value : input_channel -> 'a Lwt.t
 (** [read_value channel] reads a marshaled value from [channel]; it corresponds
     to the standard library's
-    {{:https://caml.inria.fr/pub/docs/manual-ocaml/libref/Marshal.html#VALfrom_channel} [Marshal.from_channel]}.
+    {{:https://ocaml.org/api/Marshal.html#VALfrom_channel} [Marshal.from_channel]}.
     The corresponding writing function is {!write_value}.
 
     Note that reading marshaled values is {e not}, in general, type-safe. See
     the warning in the description of module
-    {{:https://caml.inria.fr/pub/docs/manual-ocaml/libref/Marshal.html}
+    {{:https://ocaml.org/api/Marshal.html}
     [Marshal]} for details. The short version is: if you read a value of one
     type, such as [string], when a value of another type, such as [int] has
     actually been marshaled to [channel], you may get arbitrary behavior,
@@ -312,6 +317,8 @@ val write_from : output_channel -> bytes -> int -> int -> int Lwt.t
       to [oc], from [buffer] at offset [offset] and returns the number
       of bytes actually written *)
 
+val write_from_bigstring : output_channel -> Lwt_bytes.t -> int -> int -> int Lwt.t
+
 val write_from_string : output_channel -> string -> int -> int -> int Lwt.t
   (** See {!write}. *)
 
@@ -319,6 +326,8 @@ val write_from_exactly : output_channel -> bytes -> int -> int -> unit Lwt.t
   (** [write_from_exactly oc buffer offset length] writes all [length]
       bytes from [buffer] at offset [offset] to [oc] *)
 
+val write_from_exactly_bigstring : output_channel -> Lwt_bytes.t -> int -> int -> unit Lwt.t
+
 val write_from_string_exactly :
   output_channel -> string -> int -> int -> unit Lwt.t
   (** See {!write_from_exactly}. *)
@@ -327,7 +336,7 @@ val write_value :
   output_channel -> ?flags : Marshal.extern_flags list -> 'a -> unit Lwt.t
 (** [write_value channel ?flags v] writes [v] to [channel] using the [Marshal]
     module of the standard library. See
-    {{:https://caml.inria.fr/pub/docs/manual-ocaml/libref/Marshal.html#VALto_channel}
+    {{:https://ocaml.org/api/Marshal.html#VALto_channel}
     [Marshal.to_channel]} for an explanation of [?flags].
 
     The corresponding reading function is {!read_value}. See warnings about type
@@ -411,7 +420,7 @@ val open_file :
     If [~buffer] is supplied, it is used as the I/O buffer.
 
     If [~flags] is supplied, the file is opened with the given flags (see
-    {{: https://caml.inria.fr/pub/docs/manual-ocaml/libref/Unix.html#TYPEopen_flag}
+    {{: https://ocaml.org/api/Unix.html#TYPEopen_flag}
     [Unix.open_flag]}). Note that [~flags] is used {e exactly} as given. For
     example, opening a file with [~flags] and [~mode:Input] does {e not}
     implicitly add [O_RDONLY]. So, you should include [O_RDONLY] when opening
@@ -473,9 +482,9 @@ val open_temp_file :
 
     [?temp_dir] can be used to choose the directory in which the file is
     created. For the current directory, use
-    {{: https://caml.inria.fr/pub/docs/manual-ocaml/libref/Filename.html#VALcurrent_dir_name}
+    {{: https://ocaml.org/api/Filename.html#VALcurrent_dir_name}
     [Filename.current_dir_name]}. If not specified, the directory is taken from
-    {{: https://caml.inria.fr/pub/docs/manual-ocaml/libref/Filename.html#VALget_temp_dir_name}
+    {{: https://ocaml.org/api/Filename.html#VALget_temp_dir_name}
     [Filename.get_temp_dir_name]}, which is typically set to your system
     temporary file directory.
 
