@@ -1,24 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Module Lwt_process
- * Copyright (C) 2009 Jérémie Dimino
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 (** Process management *)
 
@@ -30,14 +13,14 @@ type command = string * string array
 
         {[
           ("ls", [|"ls"; "-l"|])
-        ]}.
+        ]}
 
         Notes:
 
         - if the name is the empty string, then the first argument
         will be used. You should specify a name only if you do not
         want the executable to be searched in the PATH. On Windows the
-        only way to enable automatic seach in PATH is to pass an empty
+        only way to enable automatic search in PATH is to pass an empty
         name.
 
         - it is possible to ``inline'' an argument, i.e. split it into
@@ -52,7 +35,7 @@ type command = string * string array
 
         {[
           ("", [|"echo"; "foo"; "bar"|])
-        ]}.
+        ]}
     *)
 
 val shell : string -> command
@@ -61,7 +44,9 @@ val shell : string -> command
 
 (** All the following functions take an optional argument
     [timeout]. If specified, after expiration, the process will be
-    sent a [Unix.sigkill] signal and channels will be closed. *)
+    sent a [Unix.sigkill] signal and channels will be closed. When the channels
+    are closed, any pending I/O operations on them (such as
+    {!Lwt_io.read_chars}) fail with exception {!Lwt_io.Channel_closed}. *)
 
 (** {2 High-level functions} *)
 
@@ -91,6 +76,7 @@ type redirection =
 val exec :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stdout : redirection ->
   ?stderr : redirection ->
@@ -102,24 +88,28 @@ val exec :
 val pread :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stderr : redirection ->
   command -> string Lwt.t
 val pread_chars :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stderr : redirection ->
   command -> char Lwt_stream.t
 val pread_line :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stderr : redirection ->
   command -> string Lwt.t
 val pread_lines :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stderr : redirection ->
   command -> string Lwt_stream.t
@@ -129,24 +119,28 @@ val pread_lines :
 val pwrite :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdout : redirection ->
   ?stderr : redirection ->
   command -> string -> unit Lwt.t
 val pwrite_chars :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdout : redirection ->
   ?stderr : redirection ->
   command -> char Lwt_stream.t -> unit Lwt.t
 val pwrite_line :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdout : redirection ->
   ?stderr : redirection ->
   command -> string -> unit Lwt.t
 val pwrite_lines :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdout : redirection ->
   ?stderr : redirection ->
   command -> string Lwt_stream.t -> unit Lwt.t
@@ -156,21 +150,25 @@ val pwrite_lines :
 val pmap :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stderr : redirection ->
   command -> string -> string Lwt.t
 val pmap_chars :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stderr : redirection ->
   command -> char Lwt_stream.t -> char Lwt_stream.t
 val pmap_line :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stderr : redirection ->
   command -> string -> string Lwt.t
 val pmap_lines :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stderr : redirection ->
   command -> string Lwt_stream.t -> string Lwt_stream.t
 
@@ -186,6 +184,7 @@ type state =
 class process_none :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stdout : redirection ->
   ?stderr : redirection ->
@@ -221,6 +220,7 @@ end
 val open_process_none :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stdout : redirection ->
   ?stderr : redirection ->
@@ -228,6 +228,7 @@ val open_process_none :
 val with_process_none :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stdout : redirection ->
   ?stderr : redirection ->
@@ -236,6 +237,7 @@ val with_process_none :
 class process_in :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stderr : redirection ->
   command ->
@@ -249,12 +251,14 @@ end
 val open_process_in :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stderr : redirection ->
   command -> process_in
 val with_process_in :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdin : redirection ->
   ?stderr : redirection ->
   command -> (process_in -> 'a Lwt.t) -> 'a Lwt.t
@@ -262,6 +266,7 @@ val with_process_in :
 class process_out :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdout : redirection ->
   ?stderr : redirection ->
   command ->
@@ -275,12 +280,14 @@ end
 val open_process_out :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdout : redirection ->
   ?stderr : redirection ->
   command -> process_out
 val with_process_out :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stdout : redirection ->
   ?stderr : redirection ->
   command -> (process_out -> 'a Lwt.t) -> 'a Lwt.t
@@ -288,6 +295,7 @@ val with_process_out :
 class process :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stderr : redirection ->
   command ->
 object
@@ -300,17 +308,20 @@ end
 val open_process :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stderr : redirection ->
   command -> process
 val with_process :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   ?stderr : redirection ->
   command -> (process -> 'a Lwt.t) -> 'a Lwt.t
 
 class process_full :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   command ->
 object
   inherit process_none
@@ -323,8 +334,10 @@ end
 val open_process_full :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   command -> process_full
 val with_process_full :
   ?timeout : float ->
   ?env : string array ->
+  ?cwd : string ->
   command -> (process_full -> 'a Lwt.t) -> 'a Lwt.t
