@@ -1,25 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Module Lwt_throttle
- * Copyright (C) 2008 Stéphane Glondu
- * Laboratoire PPS - CNRS Université Paris Diderot
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
+
 
 open Lwt.Infix
 
@@ -31,8 +13,6 @@ module type S = sig
   val wait : t -> key -> bool Lwt.t
 end
 
-let section = Lwt_log.Section.make "Lwt_throttle"
-
 module Make (H : Hashtbl.HashedType) : (S with type key = H.t) = struct
   module MH = Hashtbl.Make(H)
 
@@ -69,7 +49,7 @@ module Make (H : Hashtbl.HashedType) : (S with type key = H.t) = struct
           let to_run = (Queue.take elt.queue)::to_run in
           update to_run (i-1)
         with
-          | Queue.Empty -> i, 0, to_run
+        | Queue.Empty -> i, 0, to_run
     in
     let not_consumed, waiting, to_run = update to_run t.rate in
     let consumed = t.rate - not_consumed in
@@ -96,10 +76,14 @@ module Make (H : Hashtbl.HashedType) : (S with type key = H.t) = struct
         Lwt_unix.sleep 1. >>= fun () ->
         Lwt.catch
           (fun () ->
-            clean_table t;
-            Lwt.return_unit)
-          (fun exn ->
-            Lwt_log.fatal ~exn ~section "internal error")
+             clean_table t;
+             Lwt.return_unit)
+          (fun _exn ->
+             (* Not good practice, but not worse than the code it is
+                replacing. *)
+             prerr_endline "internal error";
+             Printexc.print_backtrace stderr;
+             Lwt.return ())
       in
       Some t
 
@@ -120,15 +104,15 @@ module Make (H : Hashtbl.HashedType) : (S with type key = H.t) = struct
         else (elt.consumed <- succ elt.consumed;
               Lwt.return_true)
       with
-        | Not_found ->
-          let elt = { consumed = 1;
-                      queue = Queue.create () } in
-          MH.add t.table key elt;
-          Lwt.return_true
+      | Not_found ->
+        let elt = { consumed = 1;
+                    queue = Queue.create () } in
+        MH.add t.table key elt;
+        Lwt.return_true
     in
     (match t.cleaning with
-      | None -> launch_cleaning t
-      | Some _ -> ());
+     | None -> launch_cleaning t
+     | Some _ -> ());
     res
 
 end
