@@ -1,27 +1,7 @@
-(* Lightweight thread library for OCaml
- * http://www.ocsigen.org/lwt
- * Module Lwt_timeout
- * Copyright (C) 2005-2008 Jérôme Vouillon
- * Laboratoire PPS - CNRS Université Paris Diderot
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation, with linking exceptions;
- * either version 2.1 of the License, or (at your option) any later
- * version. See COPYING file for details.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
- *)
+(* This file is part of Lwt, released under the MIT license. See LICENSE.md for
+   details, or visit https://github.com/ocsigen/lwt/blob/master/LICENSE.md. *)
+
 
-let section = Lwt_log.Section.make "lwt(timeout)"
 
 type t =
   { mutable delay : int; action : unit -> unit;
@@ -78,25 +58,24 @@ let size l =
 let handle_exn =
   ref
     (fun exn ->
-       ignore (Lwt_log.error ~section ~exn "uncaught exception after timeout");
-       exit 1)
+      !Lwt.async_exception_hook exn)
 
 let set_exn_handler f = handle_exn := f
 
 let rec loop () =
   stopped := false;
   Lwt.bind (Lwt_unix.sleep 1.) (fun () ->
-  let s = !buckets.(!curr) in
-  while not (lst_is_empty s) do
-    let x = lst_peek s in
-    decr count;
-(*XXX Should probably report any exception *)
-    try
-      x.action ()
-    with e -> !handle_exn e
-  done;
-  curr := (!curr + 1) mod (Array.length !buckets);
-  if !count > 0 then loop () else begin stopped := true; Lwt.return_unit end)
+    let s = !buckets.(!curr) in
+    while not (lst_is_empty s) do
+      let x = lst_peek s in
+      decr count;
+      (*XXX Should probably report any exception *)
+      try
+        x.action ()
+      with e -> !handle_exn e
+    done;
+    curr := (!curr + 1) mod (Array.length !buckets);
+    if !count > 0 then loop () else begin stopped := true; Lwt.return_unit end)
 
 let start x =
   let in_list = lst_in_list x in
