@@ -44,8 +44,19 @@ char *lwt_unix_strdup(char *string);
     (type *)lwt_unix_malloc(sizeof(type) + size)
 
 /* Raise [Lwt_unix.Not_available]. */
+#if OCAML_VERSION >= 40300
+CAMLnoreturn_start
+void lwt_unix_not_available(char const *feature)
+CAMLnoreturn_end;
+#else
 void lwt_unix_not_available(char const *feature) Noreturn;
+#endif
 
+#define LWT_NOT_AVAILABLE_BYTE(prim)             \
+    CAMLprim value lwt_##prim(value *a1, int a2) \
+    {                                            \
+        lwt_unix_not_available(#prim);           \
+    }
 #define LWT_NOT_AVAILABLE1(prim) \
     CAMLprim value lwt_##prim(value a1) { lwt_unix_not_available(#prim); }
 #define LWT_NOT_AVAILABLE2(prim)                  \
